@@ -25,6 +25,8 @@ from __future__ import absolute_import
 import os
 import gobject
 
+from .overrides import registry
+
 from ._gi import \
     Repository, \
     FunctionInfo, \
@@ -47,7 +49,6 @@ from .types import \
 
 repository = Repository.get_default()
 
-
 def get_parent_for_object(object_info):
     parent_object_info = object_info.get_parent()
 
@@ -85,6 +86,7 @@ class IntrospectionModule(object):
         self.__name__ = 'gi.repository.' + namespace
 
         repository.require(self._namespace, self.version)
+        self.__path__ = repository.get_typelib_path(self._namespace)
 
         if self.version is None:
             self.version = repository.get_version(self._namespace)
@@ -172,6 +174,9 @@ class IntrospectionModule(object):
         path = repository.get_typelib_path(self._namespace)
         return "<IntrospectionModule %r from %r>" % (self._namespace, path)
 
+    def __dir__ (self):
+        attribs_list = repository.get_infos(self._namespace)
+        return list(map(lambda x: x.get_name(), attribs_list))
 
 class DynamicGObjectModule(IntrospectionModule):
     """Wrapper for the GObject module
@@ -222,6 +227,7 @@ class DynamicModule(object):
 
         overrides_modules = __import__('gi.overrides', fromlist=[self._namespace])
         self._overrides_module = getattr(overrides_modules, self._namespace, None)
+        self.__path__ = repository.get_typelib_path(self._namespace)
 
     def __getattr__(self, name):
         if self.introspection_module is None:
@@ -231,5 +237,19 @@ class DynamicModule(object):
             override_exports = getattr(self._overrides_module, '__all__', ())
             if name in override_exports:
                 return getattr(self._overrides_module, name, None)
+        else:
+            # check the registry just in case the module hasn't loaded yet
+            # TODO: Only gtypes are registered in the registry right now 
+            #       but it would be nice to register all overrides and 
+            #       get rid of the module imports. We might actually see a 
+            #       speedup.
+            key = '%s.%s' % (self._namespace, name)
+            if key in registry:
+                return registry[key]
 
         return getattr(self.introspection_module, name)
+
+    def __dir__ (self):
+        repository.require(self._namespace, self._version)
+        attribs_list = repository.get_infos(self._namespace)
+        return list(map(lambda x: x.get_name(), attribs_list))
