@@ -100,7 +100,7 @@ class MetaClassHelper(object):
             for vfunc_info in base.__info__.get_vfuncs():
                 vfunc = getattr(cls, 'do_' + vfunc_info.get_name(), None)
                 if vfunc is None and isinstance(base.__info__, InterfaceInfo) and \
-                        not hasattr(cls, vfunc_info.get_name()):
+                        (not hasattr(cls, vfunc_info.get_name()) and not vfunc_info.get_invoker()):
                     raise TypeError('Class implementing %s.%s should implement '
                             'the method do_%s()' % (base.__info__.get_namespace(),
                                                     base.__info__.get_name(),
@@ -162,14 +162,6 @@ class StructMeta(type, MetaClassHelper):
         cls._setup_methods()
         cls._setup_constructors()
 
-
-def override(type_):
-    g_type = type_.__info__.get_g_type()
-    assert g_type != gobject.TYPE_NONE
-    if g_type != gobject.TYPE_INVALID:
-        g_type.pytype = type_
-    return type_
-
 class Enum(int):
     __info__ = None
     def __init__(self, value):
