@@ -349,16 +349,20 @@ pyg_type_wrapper_new(GType type)
 }
 
 /**
- * pyg_type_from_object:
+ * pyg_type_from_object_strict:
  * obj: a Python object
+ * strict: if set to TRUE, raises an exception if it can't perform the
+ *         conversion
  *
- * converts a python object to a GType.  Raises an exception if it
- * can't perform the conversion.
+ * converts a python object to a GType.  If strict is set, raises an 
+ * exception if it can't perform the conversion, otherwise returns
+ * PY_TYPE_OBJECT.
  *
  * Returns: the corresponding GType, or 0 on error.
  */
+
 GType
-pyg_type_from_object(PyObject *obj)
+pyg_type_from_object_strict(PyObject *obj, gboolean strict)
 {
     PyObject *gtype;
     GType type;
@@ -416,10 +420,35 @@ pyg_type_from_object(PyObject *obj)
     }
 
     PyErr_Clear();
+
+    /* Some API like those that take GValues can hold a python object as
+     * a pointer.  This is potentially dangerous becuase everything is 
+     * passed in as a PyObject so we can't actually type check it.  Only
+     * fallback to PY_TYPE_OBJECT if strict checking is disabled
+     */
+    if (!strict)
+        return PY_TYPE_OBJECT;
+
     PyErr_SetString(PyExc_TypeError, "could not get typecode from object");
     return 0;
 }
 
+/**
+ * pyg_type_from_object:
+ * obj: a Python object
+ *
+ * converts a python object to a GType.  Raises an exception if it
+ * can't perform the conversion.
+ *
+ * Returns: the corresponding GType, or 0 on error.
+ */
+GType
+pyg_type_from_object(PyObject *obj)
+{
+    /* Legacy call always defaults to strict type checking */
+    return pyg_type_from_object_strict(obj, TRUE);
+}
+
 /* -------------- GValue marshalling ------------------ */
 
 /**
@@ -864,8 +893,8 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
 	else if (PyObject_TypeCheck(obj, &PyGPointer_Type) &&
 		   G_VALUE_HOLDS(value, ((PyGPointer *)obj)->gtype))
 	    g_value_set_pointer(value, pyg_pointer_get(obj, gpointer));
-	else if (PyCObject_Check(obj))
-	    g_value_set_pointer(value, PyCObject_AsVoidPtr(obj));
+	else if (PYGLIB_CPointer_Check(obj))
+	    g_value_set_pointer(value, PYGLIB_CPointer_GetPointer(obj, NULL));
 	else
 	    return -1;
 	break;
@@ -910,15 +939,15 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
         }
 	else if ((bm = pyg_type_lookup(G_VALUE_TYPE(value))) != NULL)
 	    return bm->tovalue(value, obj);
-	else if (PyCObject_Check(obj))
-	    g_value_set_boxed(value, PyCObject_AsVoidPtr(obj));
+	else if (PYGLIB_CPointer_Check(obj))
+	    g_value_set_boxed(value, PYGLIB_CPointer_GetPointer(obj, NULL));
 	else
 	    return -1;
 	break;
     }
     case G_TYPE_PARAM:
 	if (PyGParamSpec_Check(obj))
-	    g_value_set_param(value, PyCObject_AsVoidPtr(obj));
+	    g_value_set_param(value, PYGLIB_CPointer_GetPointer(obj, NULL));
 	else
 	    return -1;
 	break;
